import java.util.*;
import processing.opengl.*;
import javax.media.opengl.*;
import krister.Ess.*;
static public void main(String srgs[]) {
  PApplet.main(new String[] {
    "--present", "--display=1", "ParticleWorld"    }
  );
}

ParticleSystem ps[];
EQCircle eqs[];
CircularLUT circle1,circle2;

//EES
int bufferSize;
int steps;
float limitDiff;
int numAverages = 32;
float myDamp = .1f;
float maxLimit, minLimit;
FFT myFFT;
AudioInput myInput;

//Source for Particle Size
int freqForSize = 5;

//GL Settings
PGraphicsOpenGL pgl;
GL gl;
int blendModeSrc = GL.GL_SRC_ALPHA;
int blendModeDest = GL.GL_ONE;
//World Transformation
float camZ = 0;
float camEyeZ = 500;
float camRotX,camRotY;

//Fonts
PFont font; 

//Texture
PImage tex;

//Drawing State
boolean play =true;
boolean enableEqualizerRing = false;
boolean enableHUD = false;
boolean refresh = true;
boolean autoCamRotX = false;
boolean autoCamRotY = false;
float autoCamRotXRev = 1.0;
float autoCamRotYRev = 1.0;


void setup(){
  size(1280,720,OPENGL);
  colorMode(HSB,1.0f);

  frameRate(30);
  pgl = (PGraphicsOpenGL) g;
  gl = pgl.gl;
  font = loadFont("Georgia-24.vlw");
  tex = loadImage("c.png");
  textFont(font, 16);
  circle1 = new CircularLUT(20.0f);
  circle2 = new CircularLUT(5.0f);
  play = true;

  eqs = new EQCircle[32];
  for(int i=0;i<32;i++){
    eqs[i] = new EQCircle(0.0f,0.0f,(float)i*-30.0f-100.0f,300.0f);
  }
  //FFT setup
  // start up Ess
  Ess.start(this);
  // set up AudioInput
  bufferSize = 512;
  myInput = new AudioInput(bufferSize);
  // set up FFT
  myFFT = new FFT(bufferSize * 2);
  myFFT.equalizer(true);
  // set up FFT normalization/dampening
  minLimit = .005f;
  maxLimit = .05f;
  myFFT.limits(minLimit, maxLimit);
  myFFT.damp(myDamp);
  myFFT.averages(numAverages);
  // get the number of bins per average
  steps = bufferSize / numAverages;
  // get the distance of travel between minimum and maximum limits
  limitDiff = maxLimit - minLimit;
  myInput.start();
  myFFT.damp(1.0f);

  //Colors
  FuzzColor fuzzColors[];
  fuzzColors = new FuzzColor[3];
  fuzzColors[0] = new FuzzColor(1.0f,0.2f,1.0f,0f,1f,0f);
  fuzzColors[1] = new FuzzColor(0.6f,0.1f,0.6f,0f,1f,0f);
  fuzzColors[2] = new FuzzColor(0.2f,0.1f,1f,0f,1f,0f);

  //Emitters
  Emitter emitters[];
  emitters = new Emitter[4];
  emitters[0] = new Emitter(0,0,random(0.0,5.0),5,PI/180*5,0);
  emitters[1] = new Emitter(0,0,random(0.0,5.0),10,PI/180*5,PI);
  emitters[2] = new Emitter(0,0,random(0.0,5.0),15,PI/180*10,PI);
  emitters[3] = new Emitter(0,0,random(0.0,5.0),20,PI/180*10,PI/1.5);

  Force f = new Force();

  ps = new ParticleSystem[4];
  ps[0] = new ParticleSystem(0,1,emitters[0],fuzzColors[0],f);
  ps[1] = new ParticleSystem(0,1,emitters[1],fuzzColors[1],f);
  ps[2] = new ParticleSystem(0,1,emitters[2],fuzzColors[2],f);
  ps[3] = new ParticleSystem(0,1,emitters[3],fuzzColors[1],f);

  //smooth();
  background(0,0,0);
}
void draw(){
  pgl = (PGraphicsOpenGL) g; // g may change
  gl = pgl.beginGL(); // ho ho fancy
  // This fixes the overlap issue
  gl.glDisable(GL.GL_DEPTH_TEST);
  // Turn on the blend mode
  gl.glEnable(GL.GL_BLEND);
  // Define the blend mode
  gl.glBlendFunc(blendModeSrc,blendModeDest);
  if(refresh){
    background(0,0,0);
  }
  if(autoCamRotX){
    camRotX +=autoCamRotXRev*0.5;
  }
  if(autoCamRotY){
    camRotY +=autoCamRotYRev*0.5;
  }
  camera(width/2.0f, height/2.0f, (height/2.0) / tan(PI*60.0 / 360.0), width/2.0f, height/2.0f, 0.0f, 0f, 1f, 0f);
  translate(width,height,camZ);
  rotateY(PI*camRotY/180);
  rotateX(PI*camRotX/180);
  translate(0,0,camEyeZ);


  for(int i = 0;i < ps.length;i++){
    ps[i].update();
  }
  if(enableEqualizerRing){
    equalizerRingDraw();
  }
  if(enableHUD){
    HUDDraw();
  }
  pgl.endGL(); // ho ho fancy
}
public void audioInputData(AudioInput theInput) {
  myFFT.getSpectrum(myInput);
}
void stop() {
  Ess.stop();
  super.stop();
}
void keyReleased(){
  if(keyCode == 32){
    if(play){
      play = false;
    }
    else{
      play = true;
    }
  }
  if(key == 'e'){
    if(enableEqualizerRing){
      enableEqualizerRing = false;
    }
    else{
      enableEqualizerRing =true;
    }
  }
  if(key == 'h'){
    if(enableHUD){
      enableHUD = false;
    }
    else{
      enableHUD =true;
    }
  }
  if(keyCode == RIGHT){
    freqForSize ++;
    if(freqForSize > numAverages-1){
      freqForSize =0;
    }
  }
  if(keyCode == LEFT){
    freqForSize --;
    if(freqForSize < 0){
      freqForSize =numAverages-1;
    }
  }
  if(key == '1'){
    //gl.glDisable(GL.GL_DEPTH_TEST);
    blendModeSrc = GL.GL_SRC_ALPHA;
    blendModeDest = GL.GL_ONE;
  }
  if(key == '2'){
    //gl.glEnable(GL.GL_DEPTH_TEST);
    blendModeSrc = GL.GL_ONE;
    blendModeDest = GL.GL_ZERO;
  }
  if(key == 'r'){
    if(refresh){
      refresh = false;
    }
    else{
      refresh = true;
    }
  }
  if(key == 'q'){
    if(autoCamRotX){
      autoCamRotX = false;
    }
    else{
      autoCamRotX = true;
    }
  }
  if(key == 'w'){
    if(autoCamRotY){
      autoCamRotY = false;
    }
    else{
      autoCamRotY = true;
    }
  }
  if(key == 'a'){
    if(autoCamRotXRev > 0){
      autoCamRotXRev = -1.0;
    }
    else{
      autoCamRotXRev = 1.0;
    }
  }
  if(key == 's'){
    if(autoCamRotYRev > 0){
      autoCamRotYRev = -1.0;
    }
    else{
      autoCamRotYRev = 1.0;
    }
  }
  if(key == 'g'){
      if(!online){
        String timeStamp = "_"+nf(year(),4)+nf(month(),2)+nf(day(),2)+nf(hour(),2)+nf(minute(),2)+nf(second(),2);
        save("ParticleWorld"+timeStamp+".png");
      }
  }
}
void mouseDragged(){
  if(mouseButton == RIGHT){
    camZ += (mouseY -pmouseY);
  }
  if(mouseButton == LEFT){
    camRotX -= (mouseY - pmouseY)/10.0f;
    camRotY += (mouseX - pmouseX)/10.0f;
  }
  if(mouseButton == CENTER){
    camEyeZ += (mouseY -pmouseY);
  }
}

void equalizerRingDraw(){
  for(int i=0;i<32;i++){
    eqs[i].draw(i);
  }
}

void HUDDraw(){
  gl.glEnable(GL.GL_DEPTH_TEST);
  gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
  pushMatrix();
  resetMatrix();
  stroke(0.5);
  fill(1.0,0.1);
  //camera(width/2.0f, height/2.0f, 0, width/2.0f, height/2.0f, 0.0f, 0f, 1f, 0f);
  float offsetX = width * 0.01;
  float offsetY = height* 0.02;
  translate(offsetX,offsetY);
  rect(0,0,210,130);
  fill(1.0);
  gl.glDisable(GL.GL_DEPTH_TEST);
  gl.glEnable(GL.GL_BLEND);
  textFont(font,16);
  text("ParticleWorld", 10,20);
  textFont(font,12);
  text("# Of Particles:",10,40);
  text(ps[0].particles.size()+ps[1].particles.size()+ps[2].particles.size()+ps[3].particles.size(),150,40);
  text("Equalizer",10,60);
  noStroke();
  fill(0.5,0.5,1.0,0.2);
  rect(10+6*freqForSize,120,4,0-50);
  fill(1.0,0.5);
  for(int i=0;i<numAverages;i++){
    rect(10+6*i,120,4,0-myFFT.averages[i]*50);
  }
  noStroke();
  popMatrix();
}

